@@ -33,6 +33,7 @@ import dxpy
 ASSET_BUILDER_PRECISE = "app-create_asset_precise"
 ASSET_BUILDER_TRUSTY = "app-create_asset_trusty"
 ASSET_BUILDER_XENIAL = "app-create_asset_xenial"
+ASSET_BUILDER_XENIAL_V1 = "app-create_asset_xenial_v1"
 
 
 class AssetBuilderException(Exception):
@@ -65,8 +66,9 @@ def validate_conf(asset_conf):
             "title": "A human readable name",
             "description": " A detailed description abput the asset",
             "version": "0.0.1",
-            "distribution": "Ubuntu",# (Optional)
-            "release": "12.04",
+            "runSpecVersion": "1",
+            "release": "16.04",
+            "distribution": "Ubuntu"
             "execDepends":
                         [
                             {"name": "samtools", "package_manager": "apt"},
@@ -79,22 +81,30 @@ def validate_conf(asset_conf):
     """
     if 'name' not in asset_conf:
         raise AssetBuilderException('The asset configuration does not contain the required field "name".')
-    # TODO: this default is not a good idea, and we will have to remove it once we ask customers to always provide release
-    if 'release' not in asset_conf:
-        asset_conf['release'] = "12.04"
-    elif asset_conf['release'] not in ['16.04', '14.04', '12.04']:
-        raise AssetBuilderException('The "release" field value should be either "12.04" (DEPRECATED), "14.04", "16.04".')
+
+    # Validate runSpec
+    if 'release' not in asset_conf or asset_conf['release'] not in ["16.04", "14.04", "12.04"]:
+        raise AssetBuilderException('The "release" field value should be either "16.04", "14.04", or "12.04" (DEPRECATED)')
+    if 'runSpecVersion' in asset_conf:
+        if asset_conf['runSpecVersion'] not in ["0", "1"]:
+            raise AssetBuilderException('The "runSpecVersion" field should be either "0", or "1"')
+        if (asset_conf['runSpecVersion'] == "1" and asset_conf['release'] != "16.04"):
+            raise AssetBuilderException('The "runSpecVersion" field can only be "1" if "release" is "16.04"')
+    else:
+        asset_conf['runSpecVersion'] = "0"
+    if 'distribution' in asset_conf:
+        if asset_conf['distribution'] != "Ubuntu":
+            raise AssetBuilderException('The distribution may only take the value "Ubuntu".')
+    else:
+        asset_conf['distribution'] = "Ubuntu"
+
     if 'version' not in asset_conf:
         raise AssetBuilderException('The asset configuration does not contain the required field "version". ')
     if 'title' not in asset_conf:
         raise AssetBuilderException('The asset configuration does not contain the required field "title". ')
     if 'description' not in asset_conf:
         raise AssetBuilderException('The asset configuration does not contain the required field "description".')
-    if 'distribution' in asset_conf:
-        if asset_conf['distribution'] != 'Ubuntu':
-            raise AssetBuilderException('The distribution may only take the value "Ubuntu".')
-    else:
-        asset_conf['distribution'] = "Ubuntu"
+    
 
 
 def dx_upload(file_name, dest_project, target_folder, json_out):
@@ -212,16 +222,16 @@ def build_asset(args):
         # Add the default destination project to app run options, if it is not run from a job
         if not dxpy.JOB_ID:
             builder_run_options["project"] = dest_project_name
-
         if 'instanceType' in asset_conf:
             builder_run_options["systemRequirements"] = {"*": {"instanceType": asset_conf["instanceType"]}}
         if dest_folder_name:
             builder_run_options["folder"] = dest_folder_name
-
         if asset_conf['release'] == "12.04":
             app_run_result = dxpy.api.app_run(ASSET_BUILDER_PRECISE, input_params=builder_run_options)
         elif asset_conf['release'] == "14.04":
             app_run_result = dxpy.api.app_run(ASSET_BUILDER_TRUSTY, input_params=builder_run_options)
+        elif asset_conf['release'] == "16.04" and asset_conf['runSpecVersion'] == '1':
+            app_run_result = dxpy.api.app_run(ASSET_BUILDER_XENIAL_V1, input_params=builder_run_options)
         elif asset_conf['release'] == "16.04":
             app_run_result = dxpy.api.app_run(ASSET_BUILDER_XENIAL, input_params=builder_run_options)
 
