@@ -34,7 +34,7 @@ from . import DXDataObject
 from ..exceptions import DXFileError, DXIncompleteReadsError
 from ..utils import warn
 from ..utils.resolver import object_exists_in_project
-from ..compat import BytesIO, basestring
+from ..compat import BytesIO, basestring, USING_PYTHON2
 
 
 DXFILE_HTTP_THREADS = min(cpu_count(), 8)
@@ -135,7 +135,9 @@ class DXFile(DXDataObject):
     :type dxid: string
     :param project: Project ID
     :type project: string
-    :param mode: One of "r", "w", or "a" for read, write, and append modes, respectively
+    :param mode: One of "r", "w", or "a" for read, write, and append modes, respectively.
+                 Use "b" for binary mode. For example, "rb" means open a file for reading
+                 in binary mode.
     :type mode: string
 
     .. note:: The attribute values below are current as of the last time
@@ -190,7 +192,7 @@ class DXFile(DXDataObject):
         :param project: Project ID
         :type project: string
         :param mode: One of "r", "w", or "a" for read, write, and append
-            modes, respectively
+            modes, respectively. Add "b" for binary mode.
         :type mode: string
         :param read_buffer_size: size of read buffer in bytes
         :type read_buffer_size: int
@@ -207,11 +209,18 @@ class DXFile(DXDataObject):
         :type file_is_mmapd: bool
         """
         DXDataObject.__init__(self, dxid=dxid, project=project)
+
+        # By default, a file is created in text mode. This makes a difference
+        # in python 3.
+        self._binary_mode = False
         if mode is None:
             self._close_on_exit = True
         else:
+            if 'b' in mode:
+                self._binary_mode = True
+                mode = mode.replace("b", "")
             if mode not in ['r', 'w', 'a']:
-                raise ValueError("mode must be one of 'r', 'w', or 'a'")
+                raise ValueError("mode must be one of 'r', 'w', or 'a'. Character 'b' may be used in combination (e.g. 'wb').")
             self._close_on_exit = (mode == 'w')
         self._read_buf = BytesIO()
         self._write_buf = BytesIO()
@@ -306,18 +315,37 @@ class DXFile(DXDataObject):
     def __iter__(self):
         _buffer = self.read(self._read_bufsize)
         done = False
-        while not done:
-            if b"\n" in _buffer:
-                lines = _buffer.splitlines()
-                for i in range(len(lines) - 1):
-                    yield lines[i]
-                _buffer = lines[len(lines) - 1]
-            else:
-                more = self.read(self._read_bufsize)
-                if more == b"":
-                    done = True
+        if USING_PYTHON2:
+            while not done:
+                if b"\n" in _buffer:
+                    lines = _buffer.splitlines()
+                    for i in range(len(lines) - 1):
+                        yield lines[i]
+                    _buffer = lines[len(lines) - 1]
                 else:
-                    _buffer = _buffer + more
+                    more = self.read(self._read_bufsize)
+                    if more == b"":
+                        done = True
+                    else:
+                        _buffer = _buffer + more
+        else:
+            if self._binary_mode:
+                raise DXFileError("Cannot read lines when file opened in binary mode")
+            # python3 is much stricter about distinguishing
+            # 'bytes' from 'str'.
+            while not done:
+                if "\n" in _buffer:
+                    lines = _buffer.splitlines()
+                    for i in range(len(lines) - 1):
+                        yield lines[i]
+                    _buffer = lines[len(lines) - 1]
+                else:
+                    more = self.read(self._read_bufsize)
+                    if more == "":
+                        done = True
+                    else:
+                        _buffer = _buffer + more
+
         if _buffer:
             yield _buffer
 
@@ -462,7 +490,7 @@ class DXFile(DXDataObject):
                                                   self._expected_file_size,
                                                   self._file_is_mmapd)
 
-    def write(self, data, multithread=True, **kwargs):
+    def _write2(self, data, multithread=True, **kwargs):
         '''
         :param data: Data to be written
         :type data: str or mmap object
@@ -477,6 +505,9 @@ class DXFile(DXDataObject):
             does not affect where the next :meth:`write` will occur.
 
         '''
+        if not USING_PYTHON2:
+            assert(isinstance(data, bytes))
+
         self._ensure_write_bufsize(**kwargs)
 
         def write_request(data_for_write_req):
@@ -517,6 +548,47 @@ class DXFile(DXDataObject):
             # performance when len(data) >> _write_bufsize
             self.write(data[remaining_space:], **kwargs)
 
+    def write(self, data, multithread=True, **kwargs):
+        '''
+        :param data: Data to be written
+        :type data: str or mmap object
+        :param multithread: If True, sends multiple write requests asynchronously
+        :type multithread: boolean
+
+        Writes the data *data* to the file.
+
+        .. note::
+
+            Writing to remote files is append-only. Using :meth:`seek`
+            does not affect where the next :meth:`write` will occur.
+
+        '''
+        if USING_PYTHON2:
+            self._write2(data, multithread=multithread, **kwargs)
+        else:
+            # In python3, the underlying system methods use the 'bytes' type, not 'string'
+            #
+            # This is, hopefully, a temporary hack. It is not a good idea for two reasons:
+            # 1) Performance, we need to make a pass on the data, and need to allocate
+            #    another buffer of similar size
+            # 2) The types are wrong. The "bytes" type should be visible to the caller
+            #    of the write method, instead of being hidden.
+
+            # Should we throw an exception if the file is opened in binary mode,
+            # and the data is unicode/text?
+            if isinstance(data, str):
+                bt = data.encode("utf-8")
+            elif isinstance(data, bytearray):
+                bt = bytes(data)
+            elif isinstance(data, bytes):
+                bt = data
+            elif isinstance(data, mmap.mmap):
+                bt = bytes(data)
+            else:
+                raise DXFileError("Invalid type {} for write data argument".format(type(data)))
+            assert(isinstance(bt, bytes))
+            self._write2(bt, multithread=multithread, **kwargs)
+
     def closed(self, **kwargs):
         '''
         :returns: Whether the remote file is closed
@@ -551,7 +623,10 @@ class DXFile(DXDataObject):
             # settings allow last empty part upload, try to upload
             # an empty part (otherwise files with 0 parts cannot be closed).
             try:
-                self.upload_part('', 1, **kwargs)
+                if USING_PYTHON2:
+                    self.upload_part('', 1, **kwargs)
+                else:
+                    self.upload_part(b'', 1, **kwargs)
             except dxpy.exceptions.InvalidState:
                 pass
 
@@ -576,7 +651,7 @@ class DXFile(DXDataObject):
     def upload_part(self, data, index=None, display_progress=False, report_progress_fn=None, **kwargs):
         """
         :param data: Data to be uploaded in this part
-        :type data: str or mmap object
+        :type data: str or mmap object, bytes on python3
         :param index: Index of part to be uploaded; must be in [1, 10000]
         :type index: integer
         :param display_progress: Whether to print "." to stderr when done
@@ -590,6 +665,10 @@ class DXFile(DXDataObject):
         defaults to 1. This probably only makes sense if this is the
         only part to be uploaded.
         """
+        if not USING_PYTHON2:
+            # In python3, the underlying system methods use the 'bytes' type, not 'string'
+            assert(isinstance(data, bytes))
+
         req_input = {}
         if index is not None:
             req_input["index"] = int(index)
@@ -791,7 +870,7 @@ class DXFile(DXDataObject):
             self._request_iterator = None
             raise
 
-    def read(self, length=None, use_compression=None, project=None, **kwargs):
+    def _read2(self, length=None, use_compression=None, project=None, **kwargs):
         '''
         :param length: Maximum number of bytes to be read
         :type length: integer
@@ -874,3 +953,12 @@ class DXFile(DXDataObject):
         # req = urllib2.Request(url, headers=headers)
         # response = urllib2.urlopen(req)
         # return response.read()
+
+    def read(self, length=None, use_compression=None, project=None, **kwargs):
+        data = self._read2(length=length, use_compression=use_compression, project=project, **kwargs)
+        if USING_PYTHON2:
+            return data
+        # In python3, the underlying system methods use the 'bytes' type, not 'string'
+        if self._binary_mode is True:
+            return data
+        return data.decode("utf-8")
