@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2015 DNAnexus, Inc.
+# Copyright (C) 2013-2016 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -16,7 +16,7 @@
 
 from __future__ import print_function, unicode_literals, division, absolute_import
 
-import os, sys, io, locale
+import os, sys, io, locale, threading
 from io import TextIOWrapper
 from contextlib import contextmanager
 from collections import MutableMapping
@@ -32,6 +32,8 @@ _stdio_wrapped = False
 
 if USING_PYTHON2:
     from cStringIO import StringIO
+    from httplib import BadStatusLine
+    from repr import Repr
     BytesIO = StringIO
     builtin_str = str
     bytes = str
@@ -40,6 +42,7 @@ if USING_PYTHON2:
     builtin_int = int
     int = long
     open = io.open
+    THREAD_TIMEOUT_MAX = sys.maxint
     def input(prompt=None):
         encoded_prompt = prompt.encode(sys_encoding)
         with unwrap_stream("stdin"), unwrap_stream("stdout"):
@@ -80,6 +83,8 @@ if USING_PYTHON2:
         import shlex
 else:
     from io import StringIO, BytesIO
+    from http.client import BadStatusLine
+    from reprlib import Repr
     import shlex
     builtin_str = str
     str = str
@@ -90,6 +95,7 @@ else:
     int = int
     open = open
     expanduser = os.path.expanduser
+    THREAD_TIMEOUT_MAX = threading.TIMEOUT_MAX
 
 def wrap_stdio_in_codecs():
     if USING_PYTHON2:
